/******************************************************************************
 *  IBM JEmpower                                                              *
 *                                                                            *
 *  (C) Copyright IBM Corp. 1996                                              *
 *  All Rights Reserved.                                                      * 
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 *  Module Name: jempsec.h - JEmpower Security                                *
 *                                                                            *
 *****************************************************************************/

#ifndef _JEMPSEC_H_
#define _JEMPSEC_H_

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************/

/**
 * These are security codes used by the jeCheckSecurity callback.
 * If the callback returns TRUE (non-zero), then the unsecure operation
 * is allowed to proceed.  Otherwise, a Java security exception is raised
 * Some of the codes have a string passed as a parameter.  For others,
 * the string parameter is passed as null.
 */
typedef enum tagJeSecurityCode
{
   JE_SEC_CLASSLOADER   = 1,            /* noparam */
   JE_SEC_THREAD        = 2,            /* noparam */
   JE_SEC_THREADGROUP   = 3,            /* thread group name */
   JE_SEC_EXIT          = 4,            /* exit code */
   JE_SEC_EXEC          = 5,            /* cmd to exec */
   JE_SEC_LINK          = 6,            /* lib name to link */
   JE_SEC_PROPERTIES    = 7,            /* noparam */
   JE_SEC_PROPERTY      = 8,            /* property name */
   JE_SEC_FILE_READ     = 9,            /* file name */
   JE_SEC_FILE_WRITE    = 10,           /* file name */
   JE_SEC_FD_READ       = 11,           /* noparam */
   JE_SEC_FD_WRITE      = 12,           /* noparam */
   JE_SEC_SOCKET_LISTEN = 13,           /* portnum */
   JE_SEC_SOCKET_ACCEPT = 14,           /* host:portnum */
   JE_SEC_SOCKET_CONNECT= 15,           /* fromhost->tohost */
   JE_SEC_PROTOCOL      = 16,           /* protocol name */
   JE_SEC_PACKAGE       = 17,           /* package name */
   JE_SEC_DEFINITION    = 18,           /* package name */
   JE_SEC_FACTORY       = 19,           /* noparam */
   __JE_SEC_FORCESHORT  = 32767         /* ignore. forces enum to be a short */
} jeSecurityCode;

typedef int (* JELINKAGE jeCheckSecurityFnPtr)(jeSecurityCode, char *String);
JE_FN_DECL(JERESULT, jeRegisterCheckSecurity, FP_jeRegisterCheckSecurity)
   (jeCheckSecurityFnPtr fnPtr);

#ifdef  JEMPOWER_DYNAMIC
#define JE_NAM_jeRegisterCheckSecurity "jeRegisterCheckSecurity"
#ifdef  JE_HAS_ORDINALS                
#define JE_ORD_jeRegisterCheckSecurity 1154
#endif  /* JE_HAS_ORDINALS */
#endif  /* JEMPOWER_DYNAMIC */


#ifdef __cplusplus
}
#endif

#endif
