#define APPNAME "TestApp"

#define WIN32_LEAN_AND_MEAN
// Windows Header Files:
#include <windows.h>

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>

// Local Header Files
#include "testapp.h"
#include "resource.h"
#include "..\JEmpower\JEmpower.h"

// Global Variables:

char szAppName[] = APPNAME; // The name of this application
char szTitle[]   = APPNAME": Sample Application"; // The title bar text

// Foward declarations of functions included in this code module:

JEAPPLETHND CreateApplet(JENATIVEWND hwnd, int *pHeight);

ATOM MyRegisterClass(CONST WNDCLASS*);
BOOL InitApplication(HINSTANCE);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

static void listenFunc(jobject bean, const char *method, jobject event)
{
	if (strncmp(method, "mouse", 5) != 0) {
		int x = 1;
		x++;
	}
}

static JE_CLI_FN(int) MyShowConsole(const char *string)
{
	return 0;
}


//
//  FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)
//
//  PURPOSE: Entry point for the application.
//
//  COMMENTS:
//
//      This function initializes the application and processes the
//      message loop.
//
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HANDLE hAccelTable;

	if (!hPrevInstance) {
		// Perform instance initialization:
		if (!InitApplication(hInstance)) {
			return (FALSE);
		}
	}

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) {
		return (FALSE);
	}

	hAccelTable = LoadAccelerators (hInstance, szAppName);

	jeInitJavaRuntime();
	jeInitAppletRuntime();


	jeRegisterShowConsole(MyShowConsole);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (msg.wParam);

	lpCmdLine; // This will prevent 'unused formal parameter' warnings
}



//
//  FUNCTION: InitApplication(HANDLE)
//
//  PURPOSE: Initializes window data and registers window class
//
//  COMMENTS:
//
//       In this function, we initialize a window class by filling out a data
//       structure of type WNDCLASS and calling either RegisterClass or
//       the internal MyRegisterClass.
//
BOOL InitApplication(HINSTANCE hInstance)
{
	WNDCLASS  wc;
	HWND      hwnd;

	// Win32 will always set hPrevInstance to NULL, so lets check
	// things a little closer. This is because we only want a single
	// version of this app to run at a time
	hwnd = FindWindow (szAppName, NULL);
	if (hwnd) {
		// We found another version of ourself. Lets defer to it:
		if (IsIconic(hwnd)) {
			ShowWindow(hwnd, SW_RESTORE);
		}
		SetForegroundWindow (hwnd);

		// If this app actually had any functionality, we would
		// also want to communicate any action that our 'twin'
		// should now perform based on how the user tried to
		// execute us.
		return FALSE;
	}

	// Fill in window class structure with parameters that describe
	// the main window.
	wc.style         = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc   = (WNDPROC)WndProc;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;
	wc.hInstance     = hInstance;
	wc.hIcon         = LoadIcon (hInstance, szAppName);
	wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);

	wc.lpszMenuName  = szAppName;
	wc.lpszClassName = szAppName;

	// Register the window class and return success/failure code.
	return RegisterClass(&wc);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	hWnd = CreateWindow(szAppName, szTitle, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
							  CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
							  NULL, NULL, hInstance, NULL);

	if (!hWnd) {
		return (FALSE);
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return (TRUE);
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  MESSAGES:
//
//      WM_COMMAND - process the application menu
//      WM_PAINT - Paint the main window
//      WM_DESTROY - post a quit message and return
//    WM_DISPLAYCHANGE - message sent to Plug & Play systems when the display changes
//    WM_RBUTTONDOWN - Right mouse click -- put up context menu here if appropriate
//    WM_NCRBUTTONUP - User has clicked the right button on the application's system menu
//
//
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static JEAPPLETHND hApplet=0;
	static JEBEANHND hBean = 0;
	HWND hwndApplet=0;
	static int xPos = 0, yPos = 0;
	int wmId;
	PAINTSTRUCT ps;
	HDC hdc;

	switch (message) {
		case WM_CREATE:
			/*
			 * Window initialization is performed here in WM_CREATE processing
			 * WinLoadString loads strings from the resource file.
			 */
			break;
		case WM_COMMAND:
			wmId    = LOWORD(wParam); // Remember, these are...
			//Parse the menu selections:
			switch (wmId) {
				case ID_SHOWCONSOLE:
					{
						int visible;
						jeConsoleIsVisible(&visible);
						jeConsoleSetVisible(TRUE);
						jeConsoleIsVisible(&visible);
						jeResize(hApplet,150,150);
					}
					break;
				case ID_HIDECONSOLE:
					{
						int visible;
						jeConsoleIsVisible(&visible);
						jeConsoleSetVisible(FALSE);
						jeConsoleIsVisible(&visible);
						jeAppletGetStatus(hApplet, &visible);
						jeResize(hApplet,50,50);
					}
					break;
				case ID_INITAPPLET:
					{
						HWND hwndApplet;
						int height = 0;
						hApplet = CreateApplet(hwnd, &height);
						hwndApplet = jeHWNDFromHandle(hApplet);
						SetWindowPos(hwndApplet, HWND_TOP, xPos, yPos, 0,0, SWP_NOSIZE);
						yPos += height + 10;
						jeShow(hApplet, TRUE);
						jeAppletStart(hApplet);
					}
					break;
				case ID_STARTAPPLET:
					jeAppletStart(hApplet);
					break;
				case ID_STOPAPPLET:
					jeAppletStop(hApplet);
					break;
				case ID_DESTROYAPPLET:
					jeAppletDestroy(hApplet);
					break;
				case ID_EXIT:
					PostQuitMessage(0);
					break;
				case ID_BEANCREATE:
					jeBeanInstantiate(&hBean, "test");
					break;
				case ID_BEANEMBED:
					{
						HWND bHwnd;
						jeBeanEmbed(hwnd, hBean);
						bHwnd = jeHWNDFromHandle(hBean);
						SetWindowPos(bHwnd, HWND_TOP, xPos, yPos, 150,150, SWP_SHOWWINDOW);
						yPos += 150 + 10;
					}
					break;
				case ID_BEANDISPOSE:
					jeBeanDispose(hBean);
					break;
				case ID_BEANLISTENER:
					//jeBeanAddListener(bean, listenFunc);
					MessageBoxEx(hwnd, "bogus bogus bogus", "bogus", MB_OKCANCEL, LANG_ENGLISH);
					break;
				default:
					return DefWindowProc( hwnd, message, wParam, lParam );
			} /* endswitch */
			break;

		case WM_SIZE:
			{
				static BOOL minimized = FALSE;
				int width = LOWORD(lParam);
				int height = HIWORD(lParam);

				if (wParam == SIZE_MINIMIZED) {
					jeAppletsIconify(1);
					minimized = TRUE;
				} else if (wParam == SIZE_RESTORED && minimized) {
					jeAppletsUniconify(1);
					minimized = FALSE;
				}
			}

		case WM_ERASEBKGND:
			/*
			 * Return TRUE to request PM to paint the window background
			 * in SYSCLR_WINDOW.
			 */
			return TRUE;

		case WM_PAINT:
			hdc = BeginPaint (hwnd, &ps);
			FillRect(hdc, &ps.rcPaint, GetStockObject(WHITE_BRUSH));
			EndPaint (hwnd, &ps);
			break;

		case WM_DESTROY:
			jeAppletsDestroy(1);
			PostQuitMessage(0);
			break;

		default:
			/*
			 * Everything else comes here.  This call MUST exist
			 * in your window procedure.
			 */

			return DefWindowProc( hwnd, message, wParam, lParam);
	}
	return 0;
}
